from COVID19_ES_Py import Boletim


def test_25_03():
    boletim = Boletim(
        "https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-27o-boletim-de-covid-19"
    )

    municipiosComCasos = {
        'Cachoeiro de Itapemirim': '1', 'Cariacica': '1', 'Linhares':
        '3', 'Santa Teresa': '1', 'Serra': '1', 'Vila Velha': '17', 'Vitória': '15'
    }

    dicionarioFiltrado = boletim.filtra_municipios_com_casos_confirmados()
    dicionarioFiltrado = {
        municipio: casos['casosConfirmados'] for (municipio, casos) in dicionarioFiltrado.items()
    }

    assert dicionarioFiltrado == municipiosComCasos


def test_27_03():
    boletim = Boletim(
        "https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-29o-boletim-de-covid-19"
    )

    municipiosComCasos = {
        'Cachoeiro de Itapemirim': '1', 'Cariacica': '1', 'Castelo': '1', 'Linhares': '4',
        'Santa Teresa': '1', 'São Roque do Canaã': '1', 'Serra': '5', 'Vila Velha': '21', 'Vitória': '18'
    }
    
    dicionarioFiltrado = boletim.filtra_municipios_com_casos_confirmados()
    dicionarioFiltrado = {
        municipio: casos['casosConfirmados'] for (municipio, casos) in dicionarioFiltrado.items()
    }

    assert dicionarioFiltrado == municipiosComCasos


def test_31_03():
    boletim = Boletim(
        "https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-33o-boletim-da-covid-19"
    )

    municipiosComCasos = {
        'Alto Rio Novo': '1', 'Cachoeiro de Itapemirim': '1', 'Cariacica': '5', 'Castelo': '1', 'Guarapari': '2', 'Itapemirim': '1',
        'Linhares': '4', 'Santa Teresa': '1', 'São Mateus': '1', 'São Roque do Canaã': '1', 'Serra': '14', 'Viana': '1', 'Vila Velha': '35', 'Vitória': '28'
    }
    
    dicionarioFiltrado = boletim.filtra_municipios_com_casos_confirmados()
    dicionarioFiltrado = {
        municipio: casos['casosConfirmados'] for (municipio, casos) in dicionarioFiltrado.items()
    }

    assert dicionarioFiltrado == municipiosComCasos


def test_01_04():
    boletim = Boletim(
        "https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-34o-boletim-da-covid-19"
    )

    municipiosComCasos = {
        'Afonso Cláudio': '1', 'Aracruz': '1', 'Cachoeiro de Itapemirim': '1', 'Cariacica': '7', 'Castelo': '2', 'Guarapari': '2', 'Itapemirim': '1', 'Linhares': '7', 'Santa Teresa': '1', 'São Mateus': '3', 'São Roque do Canaã': '1', 'Serra': '22', 'Viana': '1', 'Vila Velha': '36', 'Vitória': '34'
    }
    
    dicionarioFiltrado = boletim.filtra_municipios_com_casos_confirmados()
    dicionarioFiltrado = {
        municipio: casos['casosConfirmados'] for (municipio, casos) in dicionarioFiltrado.items()
    }

    assert dicionarioFiltrado == municipiosComCasos


def test_02_04():
    boletim = Boletim(
        "https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-35o-boletim-da-covid-19"
    )

    municipiosComCasos = {
        'Afonso Cláudio': '1', 'Aracruz': '1', 'Cachoeiro de Itapemirim': '1', 'Cariacica': '11', 'Castelo': '2', 'Fundão': '1', 'Guarapari': '3', 'Itapemirim': '1', 'Linhares': '7', 'Santa Teresa': '1', 'São Mateus': '3', 'São Roque do Canaã': '1', 'Serra':
        '23', 'Sooretama': '2', 'Viana': '1', 'Vila Velha': '44', 'Vitória': '36'
    }
    
    dicionarioFiltrado = boletim.filtra_municipios_com_casos_confirmados()
    dicionarioFiltrado = {
        municipio: casos['casosConfirmados'] for (municipio, casos) in dicionarioFiltrado.items()
    }

    assert dicionarioFiltrado == municipiosComCasos


def test_05_04():
    boletim = Boletim(
        "https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-38o-boletim-da-covid-19"
    )

    municipiosComCasos = {
        'Afonso Cláudio': {"casosConfirmados": '1', "casosDescartados": "3", "casosSuspeitos": "3", "totalCasos": "7", "obitos": "0"},
        'Aracruz': {'casosConfirmados': '4', 'casosDescartados': '29', 'casosSuspeitos': '6', 'totalCasos': '39', 'obitos': '0'},
        'Cachoeiro de Itapemirim': {'casosConfirmados': '1', 'casosDescartados': '35', 'casosSuspeitos': '7', 'totalCasos': '43', 'obitos': '0'},
        'Cariacica': {'casosConfirmados': '16', 'casosDescartados': '119', 'casosSuspeitos': '37', 'totalCasos': '172', 'obitos': '0'},
        'Castelo': {'casosConfirmados': '2', 'casosDescartados': '9', 'casosSuspeitos': '1', 'totalCasos': '12', 'obitos': '0'},
        'Colatina': {'casosConfirmados': '2', 'casosDescartados': '21', 'casosSuspeitos': '5', 'totalCasos': '28', 'obitos': '0'},
        'Fundão': {'casosConfirmados': '1', 'casosDescartados': '3', 'casosSuspeitos': '5', 'totalCasos': '9', 'obitos': '0'},
        'Guarapari': {'casosConfirmados': '3', 'casosDescartados': '25', 'casosSuspeitos': '2', 'totalCasos': '30', 'obitos': '0'},
        'Itapemirim': {'casosConfirmados': '1', 'casosDescartados': '5', 'casosSuspeitos': '0', 'totalCasos': '6', 'obitos': '0'},
        'Linhares': {'casosConfirmados': '8', 'casosDescartados': '89', 'casosSuspeitos': '91', 'totalCasos': '188', 'obitos': '0'},
        'Santa Teresa': {'casosConfirmados': '1', 'casosDescartados': '2', 'casosSuspeitos': '3', 'totalCasos': '6', 'obitos': '0'},
        'São Mateus': {'casosConfirmados': '4', 'casosDescartados': '24', 'casosSuspeitos': '6', 'totalCasos': '34', 'obitos': '1'},
        'São Roque do Canaã': {'casosConfirmados': '1', 'casosDescartados': '3', 'casosSuspeitos': '0', 'totalCasos': '4', 'obitos': '0'},
        'Serra': {'casosConfirmados': '31', 'casosDescartados': '176', 'casosSuspeitos': '276', 'totalCasos': '483', 'obitos': '1'},
        'Sooretama': {'casosConfirmados': '2', 'casosDescartados': '5', 'casosSuspeitos': '4', 'totalCasos': '11', 'obitos': '0'},
        'Viana': {'casosConfirmados': '1', 'casosDescartados': '17', 'casosSuspeitos': '34', 'totalCasos': '52', 'obitos': '0'},
        'Vila Velha': {'casosConfirmados': '53', 'casosDescartados': '341', 'casosSuspeitos': '161', 'totalCasos': '555', 'obitos': '2'},
        'Vitória': {'casosConfirmados': '62', 'casosDescartados': '252', 'casosSuspeitos': '58', 'totalCasos': '372', 'obitos': '2'}
    }

    assert boletim.filtra_municipios_com_casos_confirmados(
    ) == municipiosComCasos



import pytest
from bs4 import BeautifulSoup

from COVID19_ES_Py import ScraperBoletim

scraper = ScraperBoletim()


def test_success():
    URLstodosBoletins = scraper.extrai_todos_boletins()
    ultimoBoletim = scraper.carrega_ultimo_boletim()

    assert len(URLstodosBoletins) == ultimoBoletim.n



import pytest
from requests.exceptions import MissingSchema

from COVID19_ES_Py import Boletim, exceptions


# - TABELAS TIPO 1 (19_03)
def test_boletim_21():
    assert (
        Boletim(
            "https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-21o-boletim-de-covid-19"
        ).casos["Vitória"]["casosConfirmados"]
        == "5"
    )

    assert (
        Boletim(
            "https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-21o-boletim-de-covid-19"
        ).casos["Vila Velha"]["casosConfirmados"]
        == "7"
    )

    assert (
        Boletim(
            "https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-21o-boletim-de-covid-19"
        ).casos["Linhares"]["casosConfirmados"]
        == "1"
    )

    assert (
        Boletim(
            "https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-21o-boletim-de-covid-19"
        ).casos["Guarapari"]["casosConfirmados"]
        == "0"
    )


def test_boletim_22():
    assert (
        Boletim(
            "https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-22o-boletim-de-covid-19"
        ).casos["Vitória"]["casosConfirmados"]
        == "6"
    )

    assert (
        Boletim(
            "https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-22o-boletim-de-covid-19"
        ).casos["Vila Velha"]["casosConfirmados"]
        == "7"
    )

    assert (
        Boletim(
            "https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-22o-boletim-de-covid-19"
        ).casos["Linhares"]["casosConfirmados"]
        == "1"
    )

    assert (
        Boletim(
            "https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-22o-boletim-de-covid-19"
        ).casos["Guarapari"]["casosConfirmados"]
        == "0"
    )


def test_boletim_23():
    assert (
        Boletim(
            "https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-23o-boletim-de-covid-19"
        ).casos["Vitória"]["casosConfirmados"]
        == "11"
    )

    assert (
        Boletim(
            "https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-23o-boletim-de-covid-19"
        ).casos["Vila Velha"]["casosConfirmados"]
        == "11"
    )

    assert (
        Boletim(
            "https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-23o-boletim-de-covid-19"
        ).casos["Linhares"]["casosConfirmados"]
        == "2"
    )

    assert (
        Boletim(
            "https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-23o-boletim-de-covid-19"
        ).casos["Guarapari"]["casosConfirmados"]
        == "0"
    )


# TABELA TIPO 2 (22_03)
def test_boletim_24():
    assert (
        Boletim(
            "https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-24o-boletim-de-covid-19"
        ).casos["Vitoria"]["casosConfirmados"]
        == "11"
    )

    assert (
        Boletim(
            "https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-24o-boletim-de-covid-19"
        ).casos["Vila Velha"]["casosConfirmados"]
        == "11"
    )

    assert (
        Boletim(
            "https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-24o-boletim-de-covid-19"
        ).casos["Linhares"]["casosConfirmados"]
        == "2"
    )

    assert (
        Boletim(
            "https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-24o-boletim-de-covid-19"
        ).casos["Atílio Vivacqua"]["casosConfirmados"]
        == "0"
    )


# TABELA TIPO 3 (23_03)
def test_boletim_25():
    assert (
        Boletim(
            "https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-25o-boletim-de-covid-19"
        ).casos["Vitória"]["casosConfirmados"]
        == "13"
    )

    assert (
        Boletim(
            "https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-25o-boletim-de-covid-19"
        ).casos["Vila Velha"]["casosConfirmados"]
        == "12"
    )

    assert (
        Boletim(
            "https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-25o-boletim-de-covid-19"
        ).casos["Linhares"]["casosConfirmados"]
        == "2"
    )

    assert (
        Boletim(
            "https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-25o-boletim-de-covid-19"
        ).casos["Atílio Vivácqua"]["casosConfirmados"]
        == "0"
    )


def test_boletim_26():
    assert (
        Boletim(
            "https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-26o-boletim-de-covid-19"
        ).casos["Vitória"]["casosConfirmados"]
        == "15"
    )

    assert (
        Boletim(
            "https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-26o-boletim-de-covid-19"
        ).casos["Vila Velha"]["casosConfirmados"]
        == "17"
    )

    assert (
        Boletim(
            "https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-26o-boletim-de-covid-19"
        ).casos["Linhares"]["casosConfirmados"]
        == "3"
    )

    assert (
        Boletim(
            "https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-26o-boletim-de-covid-19"
        ).casos["Atílio Vivácqua"]["casosConfirmados"]
        == "0"
    )


def test_boletim_27():
    assert (
        Boletim(
            "https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-27o-boletim-de-covid-19"
        ).casos["Vitória"]["casosConfirmados"]
        == "15"
    )

    assert (
        Boletim(
            "https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-27o-boletim-de-covid-19"
        ).casos["Vila Velha"]["casosConfirmados"]
        == "17"
    )

    assert (
        Boletim(
            "https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-27o-boletim-de-covid-19"
        ).casos["Linhares"]["casosConfirmados"]
        == "3"
    )

    assert (
        Boletim(
            "https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-27o-boletim-de-covid-19"
        ).casos["Atílio Vivácqua"]["casosConfirmados"]
        == "0"
    )


def test_boletim_28():
    assert (
        Boletim(
            "https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-28o-boletim-de-covid-19"
        ).casos["Vitória"]["casosConfirmados"]
        == "17"
    )

    assert (
        Boletim(
            "https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-28o-boletim-de-covid-19"
        ).casos["Vila Velha"]["casosConfirmados"]
        == "20"
    )

    assert (
        Boletim(
            "https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-28o-boletim-de-covid-19"
        ).casos["Linhares"]["casosConfirmados"]
        == "3"
    )

    assert (
        Boletim(
            "https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-28o-boletim-de-covid-19"
        ).casos["Atílio Vivácqua"]["casosConfirmados"]
        == "0"
    )


def test_boletim_29():
    assert (
        Boletim(
            "https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-29o-boletim-de-covid-19"
        ).casos["Vitória"]["casosConfirmados"]
        == "18"
    )

    assert (
        Boletim(
            "https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-29o-boletim-de-covid-19"
        ).casos["Vila Velha"]["casosConfirmados"]
        == "21"
    )

    assert (
        Boletim(
            "https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-29o-boletim-de-covid-19"
        ).casos["Linhares"]["casosConfirmados"]
        == "4"
    )

    assert (
        Boletim(
            "https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-29o-boletim-de-covid-19"
        ).casos["Atílio Vivácqua"]["casosConfirmados"]
        == "0"
    )


def test_boletim_30():
    assert (
        Boletim(
            "https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-30o-boletim-da-covid-19"
        ).casos["Vitória"]["casosConfirmados"]
        == "20"
    )

    assert (
        Boletim(
            "https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-30o-boletim-da-covid-19"
        ).casos["Vila Velha"]["casosConfirmados"]
        == "24"
    )

    assert (
        Boletim(
            "https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-30o-boletim-da-covid-19"
        ).casos["Linhares"]["casosConfirmados"]
        == "4"
    )

    assert (
        Boletim(
            "https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-30o-boletim-da-covid-19"
        ).casos["Atílio Vivacqua"]["casosConfirmados"]
        == "0"
    )


def test_boletim_31():
    assert (
        Boletim(
            "https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-31o-boletim-da-covid-19"
        ).casos["Vitória"]["casosConfirmados"]
        == "21"
    )

    assert (
        Boletim(
            "https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-31o-boletim-da-covid-19"
        ).casos["Vila Velha"]["casosConfirmados"]
        == "28"
    )

    assert (
        Boletim(
            "https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-31o-boletim-da-covid-19"
        ).casos["Linhares"]["casosConfirmados"]
        == "4"
    )

    assert (
        Boletim(
            "https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-31o-boletim-da-covid-19"
        ).casos["Atílio Vivácqua"]["casosConfirmados"]
        == "0"
    )


def test_boletim_32():
    assert (
        Boletim(
            "https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-32o-boletim-da-covid-19"
        ).casos["Vitória"]["casosConfirmados"]
        == "26"
    )

    assert (
        Boletim(
            "https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-32o-boletim-da-covid-19"
        ).casos["Vila Velha"]["casosConfirmados"]
        == "31"
    )

    assert (
        Boletim(
            "https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-32o-boletim-da-covid-19"
        ).casos["Linhares"]["casosConfirmados"]
        == "4"
    )

    assert (
        Boletim(
            "https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-32o-boletim-da-covid-19"
        ).casos["Atílio Vivácqua"]["casosConfirmados"]
        == "0"
    )


# TABELA TIPO 4 (04_04)
def test_boletim_37():
    assert (
        Boletim(
            "https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-37o-boletim-da-covid-19"
        ).casos["Vitória"]["casosConfirmados"]
        == "43"
    )

    assert (
        Boletim(
            "https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-37o-boletim-da-covid-19"
        ).casos["Vitória"]["obitos"]
        == "2"
    )

    assert (
        Boletim(
            "https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-37o-boletim-da-covid-19"
        ).casos["Vila Velha"]["casosConfirmados"]
        == "50"
    )

    assert (
        Boletim(
            "https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-37o-boletim-da-covid-19"
        ).casos["Vila Velha"]["obitos"]
        == "2"
    )

    assert (
        Boletim(
            "https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-37o-boletim-da-covid-19"
        ).casos["Linhares"]["casosConfirmados"]
        == "8"
    )

    assert (
        Boletim(
            "https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-37o-boletim-da-covid-19"
        ).casos["Atílio Vivacqua"]["casosConfirmados"]
        == "0"
    )

    assert (
        Boletim(
            "https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-37o-boletim-da-covid-19"
        ).casos["São Mateus"]["obitos"]
        == "1"
    )


def test_boletim_38():
    assert (
        Boletim(
            "https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-38o-boletim-da-covid-19"
        ).casos["Vitória"]["casosConfirmados"]
        == "62"
    )

    assert (
        Boletim(
            "https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-38o-boletim-da-covid-19"
        ).casos["Vitória"]["obitos"]
        == "2"
    )

    assert (
        Boletim(
            "https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-38o-boletim-da-covid-19"
        ).casos["Vila Velha"]["casosConfirmados"]
        == "53"
    )

    assert (
        Boletim(
            "https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-38o-boletim-da-covid-19"
        ).casos["Vila Velha"]["obitos"]
        == "2"
    )

    assert (
        Boletim(
            "https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-38o-boletim-da-covid-19"
        ).casos["Linhares"]["casosConfirmados"]
        == "8"
    )

    assert (
        Boletim(
            "https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-38o-boletim-da-covid-19"
        ).casos["Atílio Vivacqua"]["casosConfirmados"]
        == "0"
    )

    assert (
        Boletim(
            "https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-38o-boletim-da-covid-19"
        ).casos["São Mateus"]["obitos"]
        == "1"
    )


def test_fail():
    with pytest.raises(TypeError):
        Boletim(37503003)

    with pytest.raises(MissingSchema):
        Boletim("")

    with pytest.raises(TypeError):
        Boletim(None)

    with pytest.raises(TypeError):
        Boletim(False)

    with pytest.raises(TypeError):
        Boletim(True)

    with pytest.raises(AttributeError):
        Boletim("https://coronavirus.es.gov.br/Not%C3%ADcia/")

    with pytest.raises(AttributeError):
        Boletim(
            "https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-32o-boletim-de-covid-19"
        )

    with pytest.raises(AttributeError):
        Boletim("https://www.google.com.br")

    with pytest.raises(exceptions.BoletimError):
        Boletim(
            "https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-5o-boletim-de-covid-19")



import pytest
from COVID19_ES_Py import ScraperBoletim, Boletim

scraper = ScraperBoletim()


def test_pega_dataPublicacao_formatada():
    boletimPesquisa = scraper.pesquisa_boletim_data("27/03/2020")
    assert(boletimPesquisa.pega_dataPublicacao_formatada() == "27/03/2020 19h12")

    boletimPesquisa = scraper.pesquisa_boletim_data("28/03/2020")
    assert(boletimPesquisa.pega_dataPublicacao_formatada() == "28/03/2020 19h01")

    boletimPesquisa = scraper.pesquisa_boletim_data("29/03/2020")
    assert(boletimPesquisa.pega_dataPublicacao_formatada() == "29/03/2020 19h25")


def test_pega_dataAtualizacao_formatada():
    boletimPesquisa = scraper.pesquisa_boletim_data("27/03/2020")
    assert(boletimPesquisa.pega_dataAtualizacao_formatada() == "28/03/2020 19h02")

    boletimPesquisa = scraper.pesquisa_boletim_data("28/03/2020")
    assert(boletimPesquisa.pega_dataAtualizacao_formatada() == "28/03/2020 19h14")

    boletimPesquisa = scraper.pesquisa_boletim_data("29/03/2020")
    assert(boletimPesquisa.pega_dataAtualizacao_formatada() == "30/03/2020 20h13")


def test_data_malformada():
    boletim = scraper.pesquisa_boletim_data("27/03/2020")
    boletim.dataPublicacao = None
    boletim.dataAtualizacao = None
    assert(boletim.pega_dataPublicacao_formatada() is None)
    assert(boletim.pega_dataAtualizacao_formatada() is None)

    boletim.dataPublicacao = False
    boletim.dataAtualizacao = False
    assert(boletim.pega_dataPublicacao_formatada() is None)
    assert(boletim.pega_dataAtualizacao_formatada() is None)


def test_fail():
    boletim = scraper.pesquisa_boletim_data("27/03/2020")

    with pytest.raises(AttributeError):
        boletim.dataPublicacao = True
        boletim.pega_dataPublicacao_formatada()

    with pytest.raises(AttributeError):
        boletim.dataAtualizacao = True
        boletim.pega_dataAtualizacao_formatada()

    with pytest.raises(AttributeError):
        boletim.dataPublicacao = 1
        boletim.pega_dataPublicacao_formatada()

    with pytest.raises(AttributeError):
        boletim.dataAtualizacao = 1
        boletim.pega_dataAtualizacao_formatada()



"""O módulo `boletim.py` é o principal do pacote.
Nele são introduzidas as classes e métodos utilizados para coletar dados dos boletins emitidos pelo Governo.

"""

import re
from urllib.parse import urljoin
import logging
import unicodedata
import requests
import arrow
from bs4 import BeautifulSoup
from .utils import trata_entradas_tabela, remove_caracteres_especiais, MUNICIPIOS
from .exceptions import BoletimError

DOMINIO_FEED = "https://coronavirus.es.gov.br/Noticias"
DOMINIO_BOLETINS = "https://coronavirus.es.gov.br/"

# Produz registros
logging.basicConfig(
    format="%(asctime)s %(message)s",
    datefmt="%d/%m/%Y %H:%M:%S",
    filename="scraperboletim.log",
    level=logging.DEBUG,
)
mainLogger = logging.getLogger(__name__)
mainLogger.setLevel(logging.DEBUG)


class ScraperBoletim:
    """
    Um objeto `ScraperBoletim` é capaz de varrer o feed de boletins em busca de suas URLs.

    Attributes
    ----------
    URLFeedBoletins : ``str``
        A URL do feed de boletins.
    URLsBoletins : ``list`` : ``str``
        Uma lista com URLs dos boletins (preenchida caso o método `extrai_todos_boletins` seja chamado).
    """

    def __init__(self,):
        self.URLFeedBoletins = DOMINIO_FEED
        self.URLsBoletins = None

    def carrega_html_feed(self, URLPagina):
        """Carrega o HTML de uma página do feed de boletins.

        Parameters
        ----------
        URLPagina : ``str``
            Página do feed a ser varrido.

        Returns
        ----------
        html : `BeautifulSoup` ou ``None``
            O objeto BeautifulSoup do HTML lido ou ``None`` se houver falha na requisição."""

        if URLPagina:
            req = requests.get(URLPagina)
        elif self.URLFeedBoletins:  # pragma: no cover
            req = requests.get(self.URLFeedBoletins)
        else:
            req = requests.get(
                "https://coronavirus.es.gov.br/Noticias")  # pragma: no cover
        if req.status_code == 200:
            mainLogger.info(f"Requisição bem sucedida para {req.url}")
            # Pega conteúdo (HTML) da requisição
            content = req.content
            html = BeautifulSoup(content, "html.parser")
            return html
        else:
            mainLogger.error(  # pragma: no cover
                f"Requisição falhou para {req.url}")

    def extrai_boletins_pagina(self, URLPagina=None, html=None):
        """Extrai as URLs dos boletins presentes em uma página do feed de boletins.

        Parameters
        ----------
        URLPagina : ``str``
            Página do feed a ser varrido.

        Returns
        ----------
        linksBoletins : ``list`` : ``str`` ou None
            Lista com URLs dos boletins da página do feed ou None se houver falha na requisição."""

        try:
            if not html:
                html = self.carrega_html_feed(URLPagina)
            articleNoticias = html.find_all(
                "article", class_="noticia list-content-item content-item"
            )
            linksBoletins = list(
                map(lambda article: article.find_all(
                    "a")[0]["href"], articleNoticias)
            )
            return linksBoletins
        except (AttributeError, TypeError) as err:
            raise BoletimError(
                f"Não foi possível extrair boletins da página de URL {URLPagina} ou do HTML {html}: {err}")
        except requests.exceptions.MissingSchema as err:  # pragma: no cover
            mainLogger.error(f"URL mal formada: {err}")
            raise err

    def extrai_todos_boletins(self):
        """Extrai as URLs de todos os boletins publicados até o momento.

        Returns
        ----------
        todosBoletins : ``list`` : ``str``
            Lista com URLs de todos os boletins publicados até o momento."""

        todosBoletins = []
        i = 1
        while True:
            boletinsPagina = self.extrai_boletins_pagina(
                f"https://coronavirus.es.gov.br/Noticias?page={i}"
            )
            if boletinsPagina:
                todosBoletins.extend(
                    self.extrai_boletins_pagina(
                        f"https://coronavirus.es.gov.br/Noticias?page={i}"
                    )
                )
                i += 1
            else:
                break
        self.URLsBoletins = todosBoletins
        return todosBoletins

    def url_ultimo_boletim(self, html=None):
        """Busca URL do último boletim e retorna-a.

        Returns
        ----------
        url : ``string`` ou ``None``
            URL do último boletim ou ``None`` se busca falhar."""

        if not html:
            html = self.carrega_html_feed(self.URLFeedBoletins)
        try:
            articleNoticia = html.find(
                "article", class_="noticia list-content-item content-item"
            )
            ultimaNoticia = articleNoticia.find("a")["href"]
            return urljoin(DOMINIO_BOLETINS, ultimaNoticia)
        except (TypeError, AttributeError) as err:
            mainLogger.error(f"Não foi possível ler o HTML: {err}")
            raise err

    def carrega_ultimo_boletim(self):
        """Busca a URL do último boletim e instancia objeto Boletim com esta.

        Returns
        ----------
        boletim : `Boletim`
            Objeto `Boletim` carregado com a URL do último boletim emitido."""

        urlBoletim = self.url_ultimo_boletim()
        return Boletim(urlBoletim)

    def pesquisa_boletim_data(self, data):
        """Realiza pesquisa de boletim por data de publicação.

        Parameters
        ----------
        data : ``str``
            A data de publicação do boletim a ser pesquisado.
            Formatos de data aceitos:
            "DD/MM/YYYY", "DD-MM-YYYY", "DD_MM_YYYY", "DD.MM.YYYY", "DDMMYYYY".

        Returns
        ----------
        Boletim : ``dict`` ou ``None``
            O boletim publicado na data especificada ou ``None`` se não houver."""

        dataArrow = arrow.get(
            data, ["DD/MM/YYYY", "DD-MM-YYYY",
                   "DD_MM_YYYY", "DD.MM.YYYY", "DDMMYYYY"]
        )
        dataAgora = arrow.get(
            arrow.now("America/Sao_Paulo").format("DD-MM-YYYY"), "DD-MM-YYYY"
        )

        if dataAgora >= dataArrow >= arrow.get("19/03/2020", "DD/MM/YYYY"):
            i = 1
            while True:
                req = requests.get(
                    f"https://coronavirus.es.gov.br/Noticias?page={i}")
                content = req.content
                html = BeautifulSoup(content, "html.parser")
                if req.status_code == 200:  # pragma: no cover
                    noticias = html.find_all(
                        "article", class_="noticia")

                    patternDatas = r"(\d{1,2}\/\d{1,2}\/\d{4})"
                    for noticia in noticias:
                        linkBoletim = noticia.find(
                            "h4", class_="title-list").find("a")["href"]

                        dataBoletim = noticia.find(
                            "div", class_="published").text
                        matchData = re.search(patternDatas, dataBoletim)
                        if matchData:
                            dataBoletim = arrow.get(
                                matchData.group(0), "DD/MM/YYYY")
                            if dataBoletim == dataArrow:
                                return Boletim(urljoin(DOMINIO_BOLETINS, linkBoletim))
                else:
                    mainLogger.error(  # pragma: no cover
                    f"Requisição falhou para {req.url}")
        return None


class Boletim:
    """
    Um objeto `Boletim` é capaz de extrair números de casos de um boletim.

    Parameters
    ----------
    URLBoletim : ``str``
        A URL do boletim a ser processado.

    Attributes
    ----------
    url : ``str``
        A URL do boletim.
    n : ``int``
        O número do boletim.
    titulo : ``str``
        Título do boletim.
    dataPublicacao : objeto ``Arrow``
        Objeto ``Arrow`` com a data de publicação do boletim.
    dataAtualizacao : objeto ``Arrow``
        Objeto ``Arrow`` com a data de atualização do boletim, se houver.
    corpo : ``str``
        Corpo de texto do boletim.
    html : `BeautifulSoup`
        O HTML lido como objeto BeautifulSoup.
    casos : ``dict``
        O dicionário de casos registrados nos municípios do ES.
    totalGeral : ``dict``
        O dicionário do total de casos registrados no ES.
    nMunicipiosComCasos : ``int``
        O número de municípios com algum caso, confirmado ou não.
    nMunicipiosInfectados : ``int``
        O número de municípios com algum caso confirmado.
    """

    def __init__(self, URLBoletim):
        self.url = URLBoletim
        self.n = self.extrai_numero_boletim()
        self.html = self.carrega_html_boletim()
        self.titulo = self.extrai_titulo_boletim()
        datas = self.extrai_datas_boletim()
        self.dataPublicacao = datas[0]
        self.dataAtualizacao = datas[1]
        self.corpo = self.extrai_corpo_boletim()
        self.casos = {}
        self.totalGeral = {}
        self.carrega_casos_boletim()
        self.nMunicipiosComCasos = self.conta_municipios_com_casos()
        self.nMunicipiosInfectados = self.conta_municipios_infectados()

    def __str__(self):
        return f"Boletim nº {self.n} - { self.pega_dataPublicacao_formatada()}.\nURL: {self.url}\n{self.nMunicipiosInfectados} municípios com casos confirmados.\nTotal ES: {self.totalGeral}"  # pragma: no cover

    def pega_dataPublicacao_formatada(self):
        """Formata o objeto Arrow com data de publicação para o formato usual dos boletins."""

        try:
            if self.dataPublicacao:
                return self.dataPublicacao.format("DD/MM/YYYY HH[h]mm")
        except AttributeError as err:
            mainLogger.error(f"Data {self.dataPublicacao} malformada: {err}")
            raise err

    def pega_dataAtualizacao_formatada(self):
        """Formata o objeto Arrow com data de atualização para o formato usual dos boletins."""

        try:
            if self.dataAtualizacao:
                return self.dataAtualizacao.format("DD/MM/YYYY HH[h]mm")
        except AttributeError as err:
            mainLogger.error(f"Data {self.dataAtualizacao} malformada: {err}")
            raise err

    def extrai_numero_boletim(self):
        """Extrai o número do boletim pela URL."""

        buscaPadrao = re.search(r"(\d+)o", self.url)
        if buscaPadrao:
            return int(buscaPadrao.group(1))

    def extrai_titulo_boletim(self):
        """Extrai título do HTML do boletim."""

        tituloBoletim = self.html.find("h2", class_="title-content")
        return tituloBoletim.text

    def extrai_datas_boletim(self):
        """Extrai as datas de publicação e atualização (esta se existir) do HTML do boletim."""

        datasBoletim = self.html.find("div", class_="published").text

        patternDatas = r"(\d{1,2}\/\d{1,2}\/\d{4}\s*\w+h\w*)"
        matchData = re.findall(patternDatas, datasBoletim)

        formatoData = "DD/MM/YYYY H[h]m"
        dataPublicacao = arrow.get(matchData[0], formatoData)

        try:
            dataAtualizacao = arrow.get(matchData[1], formatoData)
        except IndexError:
            dataAtualizacao = None

        return (dataPublicacao, dataAtualizacao)

    def extrai_corpo_boletim(self):
        """Extrai o corpo de texto do HTML do boletim."""

        corpo = self.html.find("div", class_="clearfix body-part")

        stringCorpo = ""
        for p in corpo.find_all("p"):
            if "13.709/2018" not in p.text:
                stringCorpo = f"{stringCorpo}\n{p.text}"
            else:
                break

        return stringCorpo

    def carrega_html_boletim(self):
        """Faz requisição para o boletim e retorna seu HTML como objeto `BeautifulSoup`."""

        req = requests.get(self.url)
        if req.status_code == 200:
            mainLogger.info(
                f"Requisição bem sucedida para {self.url}!")
            # Pega conteúdo (HTML) da requisição
            content = req.content
            html = BeautifulSoup(content, "html.parser")
            return html
        else:
            mainLogger.error(
                f"Requisição falhou para {self.url}.")

    def carrega_tabela_boletim(self):
        """Seleciona a tabela do boletim com número de casos no Estado."""

        return self.html.find(class_="clearfix body-part").find_all("tr")

    def carrega_casos_19_03(self):
        """Extrai número de casos das tabelas dos boletins tipo 1 (19/03/2020)."""

        tabela = self.carrega_tabela_boletim()

        regioes = ["Central", "Metropolitana", "Norte", "Sul"]
        # Exclui header (primeira linha) e total (última linha)
        for linha in tabela[1:-1]:
            dadosLinha = [
                elemento for elemento in list(
                    map(lambda linha: linha.text.strip(), linha.find_all("p"))
                )
                if elemento not in regioes
            ]
            municipio = dadosLinha[0]

            self.casos[municipio] = {
                "casosConfirmados": unicodedata.normalize(
                    "NFKD", dadosLinha[4]
                ).replace(" ", "0"),
                "casosDescartados": unicodedata.normalize(
                    "NFKD", dadosLinha[3]
                ).replace(" ", "0"),
                "casosSuspeitos": unicodedata.normalize(
                    "NFKD", dadosLinha[2]
                ).replace(" ", "0"),
                "totalCasos": unicodedata.normalize("NFKD", dadosLinha[1]).replace(
                    " ", "0"
                ),
            }

        dadosTotalGeral = list(
            map(
                lambda linha: unicodedata.normalize("NFKD", linha.text),
                tabela[-1].find_all("p"),
            )
        )
        self.totalGeral = {
            "casosConfirmados": unicodedata.normalize("NFKD", dadosTotalGeral[4]),
            "casosDescartados": unicodedata.normalize("NFKD", dadosTotalGeral[3]),
            "casosSuspeitos": unicodedata.normalize("NFKD", dadosTotalGeral[2]),
            "totalCasos": unicodedata.normalize("NFKD", dadosTotalGeral[1]),
        }

    def carrega_casos_22_03(self):
        """Extrai número de casos da tabela do boletim tipo 2 (22/03/2020)."""

        tabela = self.carrega_tabela_boletim()

        # Exclui header (primeira linha) e total (última linha)
        for linha in tabela[2:-2]:
            dadosLinha = list(
                map(lambda linha: linha.text.strip(), linha.find_all("td")))

            try:
                if dadosLinha[1].isdigit() and dadosLinha[0] != "Total":
                    municipio = dadosLinha[0]
                    self.casos[municipio] = {
                        "casosConfirmados": unicodedata.normalize(
                            "NFKD", dadosLinha[1]
                        ).replace(" ", "0"),
                        "casosDescartados": unicodedata.normalize(
                            "NFKD", dadosLinha[2]
                        ).replace(" ", "0"),
                        "casosSuspeitos": unicodedata.normalize(
                            "NFKD", dadosLinha[3]
                        ).replace(" ", "0"),
                        "totalCasos": unicodedata.normalize("NFKD", dadosLinha[4]).replace(
                            " ", "0"
                        ),
                    }
            except IndexError:
                pass

        dadosTotalGeral = list(
            map(
                lambda linha: unicodedata.normalize(
                    "NFKD", linha.text.strip()),
                tabela[-2].find_all("td"),
            )
        )
        self.totalGeral = {
            "casosConfirmados": unicodedata.normalize("NFKD", dadosTotalGeral[1]),
            "casosDescartados": unicodedata.normalize("NFKD", dadosTotalGeral[2]),
            "casosSuspeitos": unicodedata.normalize("NFKD", dadosTotalGeral[3]),
            "totalCasos": unicodedata.normalize("NFKD", dadosTotalGeral[4]),
        }

    def carrega_casos_23_03(self):
        """Extrai número de casos das tabelas dos boletins tipo 3 (23/03/2020)."""

        tabela = self.carrega_tabela_boletim()

        # Exclui header (primeira linha) e total (última linha)
        for linha in tabela[1:-1]:
            dadosLinha = list(
                map(lambda linha: linha.text.strip(), linha.find_all("p")))
            dadosLinha = trata_entradas_tabela(dadosLinha)

            municipio = dadosLinha[0]

            self.casos[municipio] = {
                "casosConfirmados": dadosLinha[1],
                "casosDescartados": dadosLinha[2],
                "casosSuspeitos": dadosLinha[3],
                "totalCasos": dadosLinha[4],
            }

        dadosTotalGeral = list(
            map(
                lambda linha: linha.text,
                tabela[-1].find_all("p"),
            )
        )
        self.totalGeral = {
            "casosConfirmados": dadosTotalGeral[1],
            "casosDescartados": dadosTotalGeral[2],
            "casosSuspeitos": dadosTotalGeral[3],
            "totalCasos": dadosTotalGeral[4],
        }

    def carrega_casos_04_04(self):
        """Extrai número de casos das tabelas dos boletins tipo 4 (04/04/2020)."""

        tabela = self.carrega_tabela_boletim()

        # Exclui header (primeira linha) e total (última linha)
        for linha in tabela[1:-1]:
            dadosLinha = list(
                map(lambda linha: linha.text.strip(), linha.find_all("td")))
            municipio = dadosLinha[0]

            self.casos[municipio] = {
                "casosConfirmados": unicodedata.normalize(
                    "NFKD", dadosLinha[1]
                ).replace(" ", "0"),
                "casosDescartados": unicodedata.normalize(
                    "NFKD", dadosLinha[2]
                ).replace(" ", "0"),
                "casosSuspeitos": unicodedata.normalize(
                    "NFKD", dadosLinha[3]
                ).replace(" ", "0"),
                "totalCasos": unicodedata.normalize("NFKD", dadosLinha[4]).replace(" ", "0"),
                "obitos": unicodedata.normalize(
                    "NFKD", dadosLinha[5]
                ).replace(" ", "0")
            }

        dadosTotalGeral = list(
            map(
                lambda linha: unicodedata.normalize(
                    "NFKD", linha.text.strip()),
                tabela[-1].find_all("td"),
            )
        )
        self.totalGeral = {
            "casosConfirmados": unicodedata.normalize("NFKD", dadosTotalGeral[1]),
            "casosDescartados": unicodedata.normalize("NFKD", dadosTotalGeral[2]),
            "casosSuspeitos": unicodedata.normalize("NFKD", dadosTotalGeral[3]),
            "totalCasos": unicodedata.normalize("NFKD", dadosTotalGeral[4]),
            "totalObitos": unicodedata.normalize("NFKD", dadosTotalGeral[5]),
        }

    def carrega_casos_boletim(self):
        """Carrega tabela do boletim e extrai número de casos.
        Preenche os dicionários `casos` e `totalGeral`."""

        if 21 <= self.n < 24:
            self.carrega_casos_19_03()

        elif self.n == 24:
            self.carrega_casos_22_03()

        elif 24 < self.n < 37:
            self.carrega_casos_23_03()

        elif self.n >= 37:
            self.carrega_casos_04_04()

        else:
            mainLogger.error(
                f"O boletim de nº {self.n} (n < 21, anterior a 19/03/2020) não é suportado."
            )
            raise BoletimError(
                f"O boletim de nº {self.n} (n < 21, anterior a 19/03/2020) não é suportado."
            )

    def conta_municipios_com_casos(self):
        """Retorna o número de municípios com algum caso registrado, confirmado ou não."""

        municipiosComCasos = 0
        linhasTabela = self.casos.keys()

        for elemento in linhasTabela:
            if remove_caracteres_especiais(elemento).upper() in MUNICIPIOS:
                municipiosComCasos += 1

        return municipiosComCasos

    def conta_municipios_infectados(self):
        """Retorna o número de municípios com algum caso confirmado."""

        municipiosInfectados = 0
        linhasTabela = self.casos.keys()

        for elemento in linhasTabela:
            elementoTratado = remove_caracteres_especiais(elemento).upper()
            if (
                elementoTratado in MUNICIPIOS
                and int(self.casos[elemento]["casosConfirmados"]) > 0
            ):
                municipiosInfectados += 1

        return municipiosInfectados

    def pesquisa_casos_municipio(self, municipio):
        """Realiza pesquisa no boletim por casos registrados em um município.

        Parameters
        ----------
        municipio : ``str``
            O município a ser pesquisado.

        Raises
        ----------
        `BoletimError`
            Se o município não for encontrado na lista de casos.

        Returns
        ----------
        self.casos[municipio] : ``dict``
            O dicionário de casos registrados no município."""

        stringMunicipioTratada = remove_caracteres_especiais(
            municipio).lower().strip()
        stringsMunicipiosTratadas = list(
            map(
                lambda string: remove_caracteres_especiais(
                    string).lower().strip(),
                self.casos.keys(),
            )
        )
        try:
            indiceMunicipio = stringsMunicipiosTratadas.index(
                stringMunicipioTratada)
        except ValueError:  # O município não foi encontrado
            mainLogger.error(
                f"Município {municipio} não encontrado no boletim.")
            raise BoletimError(
                f'O município "{stringMunicipioTratada}" não foi encontrado no boletim. Pode ter ocorrido um erro de digitação ou o município não registrou casos de COVID-19.'
            )
        else:
            municipio = list(self.casos.keys())[indiceMunicipio]
            return self.casos[municipio]

    def filtra_municipios_com_casos_confirmados(self):
        """Filtra o dicionário de casos por municípios com casos confirmados.

        Returns
        ----------
        municipiosComCasosConfirmados : ``dict``
            O dicionário de municípios com casos confirmados no estado."""

        municipiosComCasosConfirmados = {
            municipio: self.casos[municipio]
            for (municipio, casos) in self.casos.items()
            if int(self.casos[municipio]["casosConfirmados"]) > 0
            and remove_caracteres_especiais(municipio).upper() in MUNICIPIOS
        }

        return municipiosComCasosConfirmados



"""Scraper de boletins de casos de COVID-19 no Espírito Santo.
    >>> import COVID19_ES_Py
    >>> scraper = COVID19_ES_Py.ScraperBoletim()
    >>> boletim = scraper.carrega_ultimo_boletim()

Veja https://github.com/AtilioA/COVID19-ES-Py ou https://covid19-es-py.readthedocs.io/ para mais informações.
"""

# flake8: noqa
from .boletim import (ScraperBoletim, Boletim)

__all__ = [
    'ScraperBoletim',
    'Boletim',
]

__version__ = "1.3.0"



"""O módulo `utils.py` contém funções e constantes auxiliares."""

import unicodedata
import re


MUNICIPIOS = [
    'AFONSO CLAUDIO',
    'AGUIA BRANCA',
    'AGUA DOCE DO NORTE',
    'ALEGRE',
    'ALFREDO CHAVES',
    'ALTO RIO NOVO',
    'ANCHIETA',
    'APIACA',
    'ARACRUZ',
    'ATILIO VIVACQUA',
    'BAIXO GUANDU',
    'BARRA DE SAO FRANCISCO',
    'BOA ESPERANCA',
    'BOM JESUS DO NORTE',
    'BREJETUBA',
    'CACHOEIRO DE ITAPEMIRIM',
    'CARIACICA',
    'CASTELO',
    'COLATINA',
    'CONCEICAO DA BARRA',
    'CONCEICAO DO CASTELO',
    'DIVINO DE SAO LOURENCO',
    'DOMINGOS MARTINS',
    'DORES DO RIO PRETO',
    'ECOPORANGA',
    'FUNDAO',
    'GOVERNADOR LINDENBERG',
    'GUACUI',
    'GUARAPARI',
    'IBATIBA',
    'IBIRACU',
    'IBITIRAMA',
    'ICONHA',
    'IRUPI',
    'ITAGUACU',
    'ITAPEMIRIM',
    'ITARANA',
    'IUNA',
    'JAGUARE',
    'JERONIMO MONTEIRO',
    'JOAO NEIVA',
    'LARANJA DA TERRA',
    'LINHARES',
    'MANTENOPOLIS',
    'MARATAIZES',
    'MARECHAL FLORIANO',
    'MARILANDIA',
    'MIMOSO DO SUL',
    'MONTANHA',
    'MUCURICI',
    'MUNIZ FREIRE',
    'MUQUI',
    'NOVA VENECIA',
    'PANCAS',
    'PEDRO CANARIO',
    'PINHEIROS',
    'PIUMA',
    'PONTO BELO',
    'PRESIDENTE KENNEDY',
    'RIO BANANAL',
    'RIO NOVO DO SUL',
    'SANTA LEOPOLDINA',
    'SANTA MARIA DE JETIBA',
    'SANTA TERESA',
    'SAO DOMINGOS DO NORTE',
    'SAO GABRIEL DA PALHA',
    'SAO JOSE DO CALCADO',
    'SAO MATEUS',
    'SAO ROQUE DO CANAA',
    'SERRA',
    'SOORETAMA',
    'VARGEM ALTA',
    'VENDA NOVA DO IMIGRANTE',
    'VIANA',
    'VILA PAVAO',
    'VILA VALERIO',
    'VILA VELHA',
    'VITORIA'
]


def trata_entradas_tabela(linha):
    """ Substitui caracteres vazios de uma lista por zeros.
    Utilizada para tratar valores das colunas das tabelas dos boletins."""

    patternEntradasInvalidas = re.compile(r"\xa0|&nbsp;|^\s*$|\s*-\s*")
    linha = map(lambda coluna: re.sub(
        patternEntradasInvalidas, "0", coluna), linha)
    return list(linha)


def remove_caracteres_especiais(stringEntrada):
    """Remove caracteres especiais (acentos, etc) de uma string."""

    formaNFKD = unicodedata.normalize('NFKD', stringEntrada)
    return u"".join([c for c in formaNFKD if not unicodedata.combining(c)])



import pytest
import arrow
from COVID19_ES_Py import ScraperBoletim, Boletim, exceptions


scraper = ScraperBoletim()


def test_27_03():
    boletim = Boletim(
        "https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-29o-boletim-de-covid-19"
    )

    boletimPesquisa = scraper.pesquisa_boletim_data("27/03/2020")
    assert(boletimPesquisa.n == boletim.n)

    boletimPesquisa = scraper.pesquisa_boletim_data("27-03-2020")
    assert(boletimPesquisa.n == boletim.n)

    boletimPesquisa = scraper.pesquisa_boletim_data("27.03.2020")
    assert(boletimPesquisa.n == boletim.n)

    boletimPesquisa = scraper.pesquisa_boletim_data("27_03_2020")
    assert(boletimPesquisa.n == boletim.n)

    boletimPesquisa = scraper.pesquisa_boletim_data("27032020")
    assert(boletimPesquisa.n == boletim.n)


def test_28_03():
    boletim = Boletim(
        "https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-30o-boletim-da-covid-19"
    )

    boletimPesquisa = scraper.pesquisa_boletim_data("28-03-2020")
    assert(boletimPesquisa.n == boletim.n)


def test_29_03():
    boletim = Boletim(
        "https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-31o-boletim-da-covid-19"
    )

    boletimPesquisa = scraper.pesquisa_boletim_data("29.03.2020")
    assert(boletimPesquisa.n == boletim.n)


def test_30_03():
    boletim = Boletim(
        "https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-32o-boletim-da-covid-19"
    )

    boletimPesquisa = scraper.pesquisa_boletim_data("30_03_2020")
    assert(boletimPesquisa.n == boletim.n)


def test_31_03():
    boletim = Boletim(
        "https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-33o-boletim-da-covid-19"
    )

    boletimPesquisa = scraper.pesquisa_boletim_data("31_03_2020")
    assert(boletimPesquisa.n == boletim.n)


def test_01_04():
    boletim = Boletim(
        "https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-34o-boletim-da-covid-19"
    )

    boletimPesquisa = scraper.pesquisa_boletim_data("01_04_2020")
    assert(boletimPesquisa.n == boletim.n)


def test_too_soon():
    boletimPesquisa = scraper.pesquisa_boletim_data("01_03_2020")
    assert(boletimPesquisa is None)


def test_too_late():
    boletimPesquisa = scraper.pesquisa_boletim_data("01_03_2077")
    assert(boletimPesquisa is None)


def test_fail():
    with pytest.raises(TypeError):
        scraper.pesquisa_boletim_data(None)
    with pytest.raises(TypeError):
        scraper.pesquisa_boletim_data(True)
    with pytest.raises(TypeError):
        scraper.pesquisa_boletim_data(False)
    with pytest.raises(TypeError):
        scraper.pesquisa_boletim_data(None)
    with pytest.raises(TypeError):
        scraper.pesquisa_boletim_data(10)

    with pytest.raises(arrow.parser.ParserError):
        scraper.pesquisa_boletim_data("10")
    with pytest.raises(arrow.parser.ParserError):
        scraper.pesquisa_boletim_data("20/03")
    with pytest.raises(arrow.parser.ParserError):
        scraper.pesquisa_boletim_data("03/20")



import pytest

from COVID19_ES_Py.utils import remove_caracteres_especiais


def test_success():
    assert remove_caracteres_especiais("Afonso Cláudio") == "Afonso Claudio"
    assert remove_caracteres_especiais("Sooretama") == "Sooretama"
    assert remove_caracteres_especiais("Ibiraçu") == "Ibiracu"
    assert remove_caracteres_especiais(
        " Sao Gabriel da Palha") == " Sao Gabriel da Palha"
    assert remove_caracteres_especiais(
        "São Jose do Calçado") == "Sao Jose do Calcado"
    assert remove_caracteres_especiais("PIÚMA") == "PIUMA"
    assert remove_caracteres_especiais(
        "JERÔNIMO MONTEIRO ") == "JERONIMO MONTEIRO "
    assert remove_caracteres_especiais("GUAÇUÍ") == "GUACUI"
    assert remove_caracteres_especiais("") == ""
    assert remove_caracteres_especiais("1") == "1"
    assert remove_caracteres_especiais("_") == "_"


def test_fail():
    with pytest.raises(TypeError):
        remove_caracteres_especiais(123)
    with pytest.raises(TypeError):
        remove_caracteres_especiais(False)
    with pytest.raises(TypeError):
        remove_caracteres_especiais(True)
    with pytest.raises(TypeError):
        remove_caracteres_especiais(None)
    with pytest.raises(TypeError):
        remove_caracteres_especiais(["A"])
    with pytest.raises(TypeError):
        remove_caracteres_especiais(("A",))
    with pytest.raises(TypeError):
        remove_caracteres_especiais({"A"})



from COVID19_ES_Py import Boletim


def test_25_03():
    boletim = Boletim(
        "https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-27o-boletim-de-covid-19"
    )

    assert boletim.conta_municipios_com_casos() == 62


def test_27_03():
    boletim = Boletim(
        "https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-29o-boletim-de-covid-19"
    )

    assert boletim.conta_municipios_com_casos() == 61


def test_31_03():
    boletim = Boletim(
        "https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-33o-boletim-da-covid-19"
    )

    assert boletim.conta_municipios_com_casos() == 65


def test_01_04():
    boletim = Boletim(
        "https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-34o-boletim-da-covid-19"
    )

    assert boletim.conta_municipios_com_casos() == 64


def test_02_04():
    boletim = Boletim(
        "https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-35o-boletim-da-covid-19"
    )

    assert boletim.conta_municipios_com_casos() == 67



import pytest
from bs4 import BeautifulSoup

from COVID19_ES_Py import ScraperBoletim

scraper = ScraperBoletim()


def test_30_03():
    with open("tests/html/noticias_30_03.html", "r", encoding="utf-8") as htmlFile:
        html = BeautifulSoup(htmlFile, "html.parser")
        URLUltimoBoletim = scraper.url_ultimo_boletim(html)
        assert(URLUltimoBoletim ==
               "https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-32o-boletim-da-covid-19")


def test_31_03():
    with open("tests/html/noticias_31_03.html", "r", encoding="utf-8") as htmlFile:
        html = BeautifulSoup(htmlFile, "html.parser")
        URLUltimoBoletim = scraper.url_ultimo_boletim(html)
        assert(URLUltimoBoletim ==
               "https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-33o-boletim-da-covid-19")


def test_fail():
    with pytest.raises(TypeError):
        with open("tests/html/noticias_31_03_fail.html", "r", encoding="utf-8") as htmlFile:
            html = BeautifulSoup(htmlFile, "html.parser")
            URLUltimoBoletim = scraper.url_ultimo_boletim(html)
            print(URLUltimoBoletim)

    with pytest.raises(TypeError):
        scraper.url_ultimo_boletim("a")

    with pytest.raises(AttributeError):
        scraper.url_ultimo_boletim(1)

    with pytest.raises(AttributeError):
        scraper.url_ultimo_boletim(-1)

    with pytest.raises(AttributeError):
        scraper.url_ultimo_boletim(True)



import pytest

from COVID19_ES_Py.utils import trata_entradas_tabela


def test_success():
    # Trata caracteres "vazios" ou traços
    assert trata_entradas_tabela([""]) == ["0"]
    assert trata_entradas_tabela([" "]) == ["0"]
    assert trata_entradas_tabela(["  "]) == ["0"]
    assert trata_entradas_tabela(["   "]) == ["0"]
    assert trata_entradas_tabela(["&nbsp;"]) == ["0"]
    assert trata_entradas_tabela([u"\xa0"]) == ["0"]
    assert trata_entradas_tabela(["-"]) == ["0"]
    assert trata_entradas_tabela([" -"]) == ["0"]
    assert trata_entradas_tabela(["- "]) == ["0"]
    assert trata_entradas_tabela([" - "]) == ["0"]

    # Nenhuma mudança
    assert trata_entradas_tabela([]) == []
    assert trata_entradas_tabela(["A"]) == ["A"]
    assert trata_entradas_tabela(["Santa Teresa"]) == ["Santa Teresa"]
    assert trata_entradas_tabela(["1"]) == ["1"]
    assert trata_entradas_tabela(["0"]) == ["0"]
    assert trata_entradas_tabela(["A"]) == ["A"]
    assert trata_entradas_tabela(["Santa Teresa"]) == ["Santa Teresa"]
    assert trata_entradas_tabela(["1"]) == ["1"]
    assert trata_entradas_tabela(["0"]) == ["0"]
    assert trata_entradas_tabela(["00"]) == ["00"]
    assert trata_entradas_tabela(["Não muda"]) == ["Não muda"]
    assert trata_entradas_tabela([" Não muda"]) == [" Não muda"]
    assert trata_entradas_tabela([" Não muda "]) == [" Não muda "]


def test_fail():
    with pytest.raises(TypeError):
        trata_entradas_tabela(123)
    with pytest.raises(TypeError):
        trata_entradas_tabela(False)
    with pytest.raises(TypeError):
        trata_entradas_tabela(True)
    with pytest.raises(TypeError):
        trata_entradas_tabela(None)



from os import path
from setuptools import find_packages, setup

thisDirectory = path.abspath(path.dirname(__file__))
with open(path.join(thisDirectory, "README.md"), encoding="utf-8") as f:
    README = f.read()

setup(
    name="COVID19-ES-Py",
    version="1.3.0",
    description="Scraper de boletins de casos de COVID-19 no Espírito Santo.",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/atilioa/COVID19-ES-Py",
    author="Atílio Antônio",
    author_email="atiliodadalto@protonmail.com",
    license="GLP-3",
    classifiers=[
        "License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8"
    ],
    packages=find_packages(),
    include_package_data=True,
    install_requires=["beautifulsoup4", "requests", "arrow"],
)



import pytest
from COVID19_ES_Py import Boletim, exceptions


def test_27_03():
    boletim = Boletim(
        "https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-29o-boletim-de-covid-19"
    )

    assert(boletim.pesquisa_casos_municipio("Vitória") == {
        'casosConfirmados': '18',
        'casosDescartados': '96',
        'casosSuspeitos': '142',
        'totalCasos': '256'
    })
    assert(boletim.pesquisa_casos_municipio("  santa teresa ") == {
           'casosConfirmados': '1',
           'casosDescartados': '1',
           'casosSuspeitos': '0',
           'totalCasos': '2'
           })
    assert(boletim.pesquisa_casos_municipio("AFONSO CLAUDIO") == {
           'casosConfirmados': '0',
           'casosDescartados': '1',
           'casosSuspeitos': '0',
           'totalCasos': '1'
           })


def test_29_03():
    boletim = Boletim(
        "https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-31o-boletim-da-covid-19"
    )

    assert(boletim.pesquisa_casos_municipio("Vitória") == {
        'casosConfirmados': '21',
        'casosDescartados': '133',
        'casosSuspeitos': '117',
        'totalCasos': '271'
    })
    assert(boletim.pesquisa_casos_municipio("  santa teresa ") == {
           'casosConfirmados': '1',
           'casosDescartados': '1',
           'casosSuspeitos': '0',
           'totalCasos': '2'
           })
    assert(boletim.pesquisa_casos_municipio("AFONSO CLAUDIO") == {
           'casosConfirmados': '0',
           'casosDescartados': '1',
           'casosSuspeitos': '0',
           'totalCasos': '1'
           })


def test_30_03():
    boletim = Boletim(
        "https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-32o-boletim-da-covid-19"
    )

    assert(boletim.pesquisa_casos_municipio("Vitória") == {
        'casosConfirmados': '26',
        'casosDescartados': '139',
        'casosSuspeitos': '118',
        'totalCasos': '283'
    })
    assert(boletim.pesquisa_casos_municipio("  santa teresa ") == {
           'casosConfirmados': '1',
           'casosDescartados': '1',
           'casosSuspeitos': '0',
           'totalCasos': '2'
           })
    assert(boletim.pesquisa_casos_municipio(" SAO JOSE DO CALCADO") == {
           'casosConfirmados': '0',
           'casosDescartados': '0',
           'casosSuspeitos': '1',
           'totalCasos': '1'
           })


def test_31_03():
    boletim = Boletim(
        "https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-33o-boletim-da-covid-19"
    )

    assert(boletim.pesquisa_casos_municipio("Vitória") == {
        'casosConfirmados': '28',
        'casosDescartados': '155',
        'casosSuspeitos': '121',
        'totalCasos': '304'
    })

    assert(boletim.pesquisa_casos_municipio("  santa teresa ") == {
           'casosConfirmados': '1',
           'casosDescartados': '1',
           'casosSuspeitos': '0',
           'totalCasos': '2'
           })
    assert(boletim.pesquisa_casos_municipio(" AFONSO CLAUDIO") == {
           'casosConfirmados': '0',
           'casosDescartados': '1',
           'casosSuspeitos': '3',
           'totalCasos': '4'
           })


def test_fail_24_03():
    boletim = Boletim(
        "https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-26o-boletim-de-covid-19"
    )

    # Município não é do ES
    with pytest.raises(exceptions.BoletimError):
        boletim.pesquisa_casos_municipio("arapiraca")

    # Nome de município incompleto
    with pytest.raises(exceptions.BoletimError):
        boletim.pesquisa_casos_municipio("vi")

    # Município ainda não havia caso registrado
    with pytest.raises(exceptions.BoletimError):
        boletim.pesquisa_casos_municipio("Pedro Canário")



import pytest
import requests
from bs4 import BeautifulSoup

from COVID19_ES_Py import ScraperBoletim, exceptions

scraper = ScraperBoletim()


def test_30_03():
    with open("tests/html/noticias_30_03.html", "r", encoding="utf-8") as htmlFile:
        html = BeautifulSoup(htmlFile, "html.parser")
        boletinsPagina = scraper.extrai_boletins_pagina(html=html)
        assert(boletinsPagina == [
            'https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-32o-boletim-da-covid-19',
            'https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-31o-boletim-da-covid-19',
            'https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-30o-boletim-da-covid-19',
            'https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-29o-boletim-de-covid-19',
            'https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-28o-boletim-de-covid-19',
            'https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-27o-boletim-de-covid-19',
            'https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-26o-boletim-de-covid-19',
            'https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-25o-boletim-de-covid-19',
            'https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-24o-boletim-de-covid-19',
            'https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-23o-boletim-de-covid-19'
        ])


def test_31_03():
    with open("tests/html/noticias_31_03.html", "r", encoding="utf-8") as htmlFile:
        html = BeautifulSoup(htmlFile, "html.parser")
        boletinsPagina = scraper.extrai_boletins_pagina(html=html)
        assert(boletinsPagina == [
            'https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-33o-boletim-da-covid-19',
            'https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-32o-boletim-da-covid-19',
            'https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-31o-boletim-da-covid-19',
            'https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-30o-boletim-da-covid-19',
            'https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-29o-boletim-de-covid-19',
            'https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-28o-boletim-de-covid-19',
            'https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-27o-boletim-de-covid-19',
            'https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-26o-boletim-de-covid-19',
            'https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-25o-boletim-de-covid-19',
            'https://coronavirus.es.gov.br/Not%C3%ADcia/secretaria-da-saude-divulga-24o-boletim-de-covid-19',
        ])


def test_fail():
    with pytest.raises(exceptions.BoletimError):
        scraper.extrai_boletins_pagina(html="a")

    with pytest.raises(exceptions.BoletimError):
        scraper.extrai_boletins_pagina(html=5)

    with pytest.raises(requests.exceptions.MissingSchema):
        scraper.extrai_boletins_pagina(URLPagina="a")

    with pytest.raises(requests.exceptions.MissingSchema):
        scraper.extrai_boletins_pagina(URLPagina=2)

    with pytest.raises(requests.exceptions.MissingSchema):
        scraper.extrai_boletins_pagina(URLPagina=True)

    with pytest.raises(exceptions.BoletimError):
        scraper.extrai_boletins_pagina(html=True)
